/*
 * util.c --  A couple of low level utilities needed for
 *            the Problem Tracking System (PTS) database.
 * Dean Collins 
 * created: Tue Oct 20 10:13:14 PDT 1992
 */

/*
 * Copyright (c) 1992,1993 Dean Collins.
 * Copyright (c) 1992 University of Idaho, Moscow, Idaho.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation free of charge for any purpose is hereby granted without
 * fee, provided that the above copyright notices appear in all copies and
 * that both those copyright notices and this permission notice appear in
 * supporting documentation, and that neither the name of the University of
 * Idaho nor the name of Dean Collins be used in advertising or publicity
 * pertaining to distribution of the software without specific, written
 * prior permission from both parties.  Neither The University of Idaho
 * nor Dean Collins make any representations about the suitability of
 * this software for any purpose.  It is provided "as is" without express
 * or implied warranty.
 * 
 * THE UNIVERSITY OF IDAHO AND DEAN COLLINS DISCLAIM ALL WARRANTIES WITH
 * REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL THE UNIVERSITY OF IDAHO
 * OR DEAN COLLINS BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 */

#include <stdlib.h>             /* General C utilities header file */
#include <stdio.h>		/* Standard input/output utilities hdr. file */
#include <sys/types.h>          /* Common data types */
#include <sys/stat.h>           /* File status utilities */
#include <errno.h>              /* Error utilities header file */

#include "zdbm.h"               /* Zombie Database Manager header file */
#include "cloud.h"		/* Nebulous Cloud header file */
#include "clouderror.h"         /* Nebulous Cloud error rtn. header file */
#include "config.h"             /* Config. file parser header file */

#ifdef XPTS
#include <X11/Intrinsic.h>      /* Include standard Toolkit Header file. */
#include <X11/StringDefs.h>     /* Include StringDefs header file */
#include <X11/Shell.h>          /* Include the Shell header file for popups */
#include "xpts.h"               /* Xpts header file */
extern char Path[] ;
#else
#include "defines.h"
#include "util.h"
char Path[1024]; 		/*Not really used*/
#endif

#define WORLD_WRITE  002
#define GROUP_WRITE  020
#define OWNER_WRITE 0200
#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif




/******************** MODULES *************************************/
/*
______________________________________________________________________
FileExists()

FUNCTION DESCRIPTION:

     This function determines whether or not a file exists.
______________________________________________________________________
UNIT TESTING:
     This function will be tested by using a combination of white-box and
black-box tests.
______________________________________________________________________
REVISION HISTORY:
     Author:  Dean Collins      Date:  10/20/92
______________________________________________________________________
*/


int
FileExists(char *fname)

     /* Interface description:
         INPUT:
	  fname - The name of the file to search for.
         OUTPUT:
          None.
         RETURN VALUE:
          int   - TRUE if file fname exists, FALSE otherwise.
     */
      
{
     /* Internal variables: */
          /* Major */
   struct stat fileinfo ;  /* Status information record for stat() */
          /* Minor */
   /* NONE */
   int status=0;


   /*------------------- Start of FileExists() routine --------------------*/
#ifdef DEBUG
   printf("XPTS(util.c): Entering FileExists()\n") ;
#endif

     /* Get the information about the file using stat() */
   status=stat(fname, &fileinfo) ;
#ifdef DEBUG
   printf("XPTS(util.c): status=%d\n",status) ;
#endif

     /* If the file exists, return TRUE 
      * else return FALSE 
      */

   if (status == 0)
   {
#ifdef DEBUG
      printf("XPTS(util.c): %s exists.\n", fname) ;
      printf("XPTS(util.c): Leaving FileExists()\n") ;
#endif
      return (TRUE) ;
   }

#ifdef DEBUG
   printf("XPTS(util.c): %s does not exist.\n", fname) ;
   printf("XPTS(util.c): Leaving FileExists()\n") ;
#endif
   return(FALSE) ;

}  /*----------------------- End of FileExists() --------------------------*/


/*
______________________________________________________________________
IsWritable()

FUNCTION DESCRIPTION:

     This function determines whether or not a file is writable
by the current user.
______________________________________________________________________
UNIT TESTING:
     This function will be tested by using a combination of white-box and
black-box tests.
______________________________________________________________________
REVISION HISTORY:
     Author:  Dean Collins      Date:  10/20/92
______________________________________________________________________
*/


int 
IsWritable(char *fname)

     /* Interface description:
         INPUT:
	  fname - The name of the file to search for.
         OUTPUT:
          None.
         RETURN VALUE:
          int   - TRUE if file is writable, FALSE otherwise.
     */
      
{
     /* Internal variables: */
          /* Major */
   struct stat fileinfo ;  /* Status information record for stat() */

          /* Minor */
   int err ;


   /*------------------- Start of IsWritable() routine --------------------*/
#ifdef DEBUG
   printf("XPTS(util.c): Entering IsWritable()\n") ;
#endif

     /* Get the information about the file using stat() */
   err = stat (fname, &fileinfo) ;


     /* If the file doesn't exist, return FALSE */
   if (err != 0)
   {  
#ifdef DEBUG
      printf ("Error obtaining status info of %s\n", fname) ;
      printf("XPTS(util.c): Leaving IsWritable()\n") ;
#endif
      return(FALSE) ;
   }


     /* If the file is world writable, return TRUE */
   if (fileinfo.st_mode & WORLD_WRITE)
   {  
#ifdef DEBUG
      printf("XPTS(util.c): %s is world-writable.\n",fname) ;
      printf("XPTS(util.c): Leaving IsWritable()\n") ;
#endif
      return (TRUE) ;
   }

     /* If the file is writable by the user's group, return TRUE */
   if ((fileinfo.st_mode & GROUP_WRITE) && (getegid() == fileinfo.st_gid))
   {  
#ifdef DEBUG
      printf("XPTS(util.c): %s is group-writable.\n",fname) ;
      printf("XPTS(util.c): Leaving IsWritable()\n") ;
#endif
      return (TRUE) ;
   }

     /* If the file is writable by the user, return TRUE */
   if ((fileinfo.st_mode & OWNER_WRITE) && (geteuid() == fileinfo.st_uid))
   {  
#ifdef DEBUG
      printf("XPTS(util.c): %s is owner-writable.\n",fname) ;
      printf("XPTS(util.c): Leaving IsWritable()\n") ;
#endif
      return (TRUE) ;
   }


     /* Otherwise, return FALSE.  The file is not writable by the user. */

#ifdef DEBUG
   printf("XPTS(util.c): %s is not writable.\n",fname) ;
   printf("XPTS(util.c):  Leaving IsWritable()\n") ;
#endif
   return(FALSE) ;

}  /*----------------------- End of IsWritable() --------------------------*/

/*
______________________________________________________________________
FormatProblem()

FUNCTION DESCRIPTION:

     This function will format a problem_record into a single string
that can be displayed in an Athena text widget.  (This might really
belong in The Cloud, but, being in charge of the user-interface, 
I needed control of how problems were formatted for displaying.
This should be straightened out, since there's a duplicate of this
function in the cloud for printing.)
______________________________________________________________________
REVISION HISTORY:
     Author:  Dean Collins			Date:  9/07/92
     Version 1.0 finalized: Dean Collins	Date: 1/9/93
______________________________________________________________________
*/


char *
FormatProblem(problem_record *problem)

     /* Interface description:
         INPUT:
          problem	- The problem to format.
         OUTPUT:
          None.
         RETURN:
          char *	- The formatted problem.
     */
{
     /* Internal variables: */
          /* Major */
      /* NONE */
          /* Minor */
      reporter_record *reporter=NULL ;
      int num_rep ;
      int i,j,buflen ;
      char tempstr[DBPATHLEN] ;
/*      static char *probtext=NULL ;*/
      static char probtext[MAXTEXT+1] ;/* I'm a cheater! */
      int status ;
      reopened_record *ReopenedRec=NULL ;


   /*------------ Start of FormatProblem() routine -------------*/
#ifdef DEBUG
   printf("XPTS(util.c): Entering FormatProblem()\n") ;
#endif

   if (problem==NULL)
      return(NULL) ;

   status = problem->status ;

   if (status == REOPENED)
   {  /* Assign ReopenedRec and fill in Path from it */
      ReopenedRec=(reopened_record *)(problem->reop_rec) ;
      strcpy(Path, ReopenedRec->db_path) ;
   }

#ifdef DEBUG
   printf("XPTS(util.c): FormatProblem() -- ProblemRecord->status = %d\n",
	status) ;
#endif

     /* Zero out the probtext string. */
   probtext[0] = '\0' ;

     /* Show the problem type */

   strcat(probtext, "PROBLEM TYPE:\t") ;
   if (strncmp(ROOTSTR,Path,strlen(ROOTSTR)) == 0)
/*CLD*/      strcat(probtext, FixPath(Path)) ;
   else strcat(probtext, Path) ;

     /* Show the problem status */
   strcat(probtext, "\nSTATUS:\t\t") ;
   if      (status == SOLVED)   strcat(probtext, "Solved\n\n") ;
   else if (status == UNSOLVED) strcat(probtext, "Unsolved\n\n") ;
   else if (status == REOPENED) strcat(probtext, "Reopen Request\n\n") ;
   else
   {
#     ifdef DEBUG
      printf("XPTS(util.c): Error in problem status!\n") ;
#     endif
      strcat(probtext, "(Unknown)\n\n") ;
   }

     /* Add summary */
   strcat(probtext, "SUMMARY:\n") ;
   strcat(probtext, problem->short_description) ;
   strcat(probtext, "\n\n") ;

     /* Add the reporters */
	/* Add column headers */
   sprintf(tempstr, "%-*s%-*s%s\n", NAMELEN, "REPORTER", NODELEN, "NODE",
          "DATE/TIME") ;
   strcat(probtext, tempstr) ;
   sprintf(tempstr, "%-*s%-*s%s\n", NAMELEN, "--------", NODELEN, "----",
          "---------") ;
   strcat(probtext, tempstr) ;

	/* Add reporters list */
   if (status == SOLVED)
   {  reporter = problem->old_reporter_list ;
      num_rep = problem->num_old_rep ;
   }
   else if (status == UNSOLVED)
   {  reporter = problem->reporter_list ;
      num_rep = problem->num_rep ;
   }
   else if (status == REOPENED)
   {  reporter = &(ReopenedRec->rep_rec) ;
      num_rep = 1 ;
   }

   if (reporter != NULL)
      for (i=0; i<num_rep; ++i)
      {  /* Add the reporter name */
#ifdef DEBUG
         printf("XPTS(util.c): appending reporter record:\n") ;
         printf("\treporter\t- %s\n", (reporter[i]).account_name) ;
         printf("\tnode\t- %s\n", (reporter[i]).node_name) ;
         printf("\tdate\t- %s\n", (reporter[i]).date_time) ;
#endif
         strncat(probtext, (reporter[i]).account_name, NAMELEN) ;
         buflen=strlen((reporter[i]).account_name) ;
         for (j=buflen; j<NAMELEN; ++j)
            strcat(probtext, " ") ;

         /* Add the reporter's node */
         strncat(probtext, (reporter[i]).node_name, NODELEN) ;
         buflen=strlen((reporter[i]).node_name) ;
         for (j=buflen; j<NODELEN; ++j)
            strcat(probtext, " ") ;
   
         /* Add the reporting date */
         strcat(probtext, (reporter[i]).date_time) ;
         j=strlen((reporter[i]).date_time) ;
         if (j)
            if (((reporter[i]).date_time)[j-1] == '\n')
               probtext[strlen(probtext)-1] = '\0' ;


         /* Add EOL */
         strcat(probtext, "\n") ;
      }
#ifdef DEBUG
   else printf("XPTS(util.c): FormatProblem():  reporters list was NULL!\n") ;
#endif
   strcat(probtext, "\n") ;

     /* Add problem log */
   if (status == REOPENED)
   {  strcat(probtext, REOPEN_REQ_STR) ;
      strcat(probtext, ReopenedRec->reasons_log) ;
      strcat(probtext, REOPEN_REQ_STR2) ;
   }
   strcat(probtext, PROBLEM_STR) ;
   strcat(probtext, problem->log_file) ;
   strcat(probtext, "\n") ;

#ifdef DEBUG
   printf("---------------------FORMATTED PROBLEM---------------------\n") ;
   printf("%s", probtext);
   printf("-----------------END-FORMATTED PROBLEM---------------------\n") ;
   printf("XPTS(util.c): Leaving FormatProblem()\n") ;
#endif

   return(probtext) ;

}  /*-------- End of FormatProblem() ---------------------------*/


/*
______________________________________________________________________
FixStr()

FUNCTION DESCRIPTION:

     This function "fixes" strings so they don't contain new-lines
and strips off leading and trailing white-space.
______________________________________________________________________
REVISION HISTORY:
     Author:  Dean Collins      Date:  2/13/93
______________________________________________________________________
*/


char *FixStr(char *str)

     /* Interface description:
         INPUT:
          str    - The string to fix.
         OUTPUT:
          None.
         RETURN VALUE:
          char * - The fixed string.  This is static data, and must
			be copied to be used.
     */

{
     /* Internal variables: */
          /* Major */
	  /* NONE */
          /* Minor */
   static char *tmpstr=NULL ;
   char *tmpstr2 ;
   int i ;


   /*------------------- Start of FixStr() routine --------------------*/

#ifdef DEBUG
   printf("XPTS(util.c): FixStr() str(before)='%s'\n",str) ;
#endif

   if ( strlen(str) == 0 ) return ;

   if (tmpstr) free(tmpstr) ;
   tmpstr = malloc(strlen(str)) ;
   strcpy(tmpstr,str) ;

   for (i=0; tmpstr[i] != '\0'; ++i)    /* Nuke newlines */
      if (tmpstr[i] == '\n')
         tmpstr[i] = ' ' ;

   tmpstr2 = StripLeading(StripTrailing(tmpstr)) ;

#ifdef DEBUG
   printf("XPTS(util.c): FixStr() str(after)='%s'\n",tmpstr2) ;
#endif

   return(tmpstr2) ;

}  /*-------- End of FixStr() ---------------------------*/

/*
______________________________________________________________________
StripLeading()

FUNCTION DESCRIPTION:

     This function removes leading spaces and tabs from a string.
______________________________________________________________________
REVISION HISTORY:
     Author:  Dean Collins      Date:  2/13/93
______________________________________________________________________
*/


char *StripLeading(char *str)

     /* Interface description:
         INPUT:
          str    - The string to strip.
         OUTPUT:
          None.
         RETURN VALUE:
          char * - The stripped string.
     */

{
     /* Internal variables: */
          /* Major */
          /* NONE */
          /* Minor */
   char *c ;
   int i ;


   /*---------------- Start of StripLeading() routine -----------------*/

#ifdef DEBUG
   printf("XPTS(util.c): StripLeading() str(before)='%s'\n",str) ;
#endif

        /* strip leading spaces.  c will point to the first non-space
         * and non-tab character in tmpstr.
         */
   for (c=str,i=0;
                (str[i]==' ' || str[i]=='\t') && i<strlen(str) ;
                ++c, ++i)
        ; /*nothing*/

#ifdef DEBUG
   printf("XPTS(util.c): StripLeading() str(after)='%s'\n",c) ;
#endif

   return(c) ;

}  /*----------End of StripLeading() --------------------*/


/*
______________________________________________________________________
StripTrailing()

FUNCTION DESCRIPTION:

     This function removes trailing spaces and tabs from a string.
______________________________________________________________________
REVISION HISTORY:
     Author:  Dean Collins      Date:  2/13/93
______________________________________________________________________
*/


char *StripTrailing(char *str)

     /* Interface description:
         INPUT:
          str    - The string to strip. 
         OUTPUT:
          None.
         RETURN VALUE:
          char * - The stripped string.  This data is static and must
		   be copied to be used.
     */

{
     /* Internal variables: */
          /* Major */
          /* NONE */
          /* Minor */
   static char *tmpstr=NULL ;
   int i ;


   /*---------------- Start of StripTrailing() routine -----------------*/

   if (tmpstr) free(tmpstr) ;
   tmpstr = malloc(strlen(str)) ;
   strcpy(tmpstr,str) ;

#ifdef DEBUG
   printf("XPTS(util.c): StripTrailing() str(before)='%s'\n",tmpstr) ;
#endif
        /* strip trailing spaces and tabs */
   for (i=strlen(tmpstr)-1;
                i>=0 && (tmpstr[i]==' ' || tmpstr[i]=='\t');
                tmpstr[i--]='\0')
        ; /*nothing*/

#ifdef DEBUG
   printf("XPTS(util.c): StripTrailing() str(after)='%s'\n",tmpstr) ;
#endif

   return(tmpstr) ;

}  /*----------End of StripTrailing() --------------------*/

/* end of util.c */
