/*
 * reopen_problem.c -- 
 * Chris Hyde
 * created: 
 */

/*
 * Copyright (c) 1992 Christopher G. Hyde
 * Copyright (c) 1992 University of Idaho, Moscow, Idaho.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation free of charge for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the University of Idaho
 * not be used in advertising or publicity pertaining to distribution of
 * the software without specific, written prior permission.  The University
 * of Idaho makes no representations about the suitability of this software
 * for any purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE UNIVERSITY OF IDAHO DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE UNIVERSITY OF IDAHO BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "zdbm.h"


int reopen_request(char *db_path, char *prid, char *reasons,
		   reporter_record *rep_rec)
{
  index_record index,
	       prindex;
  long num_rec;
  FILE *temp,
       *itemp;
  char path[DBPATHLEN];
  char tprid[MAXPRID];

#ifdef DEBUG
 printf("\nIn reopen_request()\n");
 printf("\tPath of problem to reopen - %s/%s\n", db_path, prid);
#endif

  db_errorno=0;

/* open the index of the problem to reopen */
 if ((itemp=open_file(db_path, ".index", "r+", INDEX))==NULL)
    return(FAIL); /* the open failed */

 if (find_problem(itemp, prid, &prindex)==NULL) /* find problem to reopen */
 {  /* error searching for problem */
    close_file(itemp);
    return(FAIL);
 }

#ifdef DEBUG
 printf("reopen_request() - found problem to reopen\n");
#endif

 if (prindex.sf!=SOLVED) /* can only reopen SOLVED problems */
 {
    db_errorno=ISUNSOLVED;
    close_file(itemp);
    return(FAIL);
 }
 if (prindex.mrd==GONE) /* can't reopen a deleted problem */
 {
    db_errorno=DELETED;
    close_file(itemp);
    return(FAIL);
 }
 if (prindex.mrd==LINKED) /* the problem was linked to the leaf given */
 {
#ifdef DEBUG
 printf("reopen_request() - problem to reopen is a link\n");
#endif

/* since we are setting the mrd field of records that are requested for
 * reopening we need to have itemp pointing to the original index entry if
 * the problem given is a link
 */
    nl_strncpy(path, prindex.ln_path, DBPATHLEN); /* path is used to access 
						* the records */
    nl_strncpy(tprid, prindex.ln_prid, MAXPRID); /* tprid is used to access 
					       * the records */

    close_file(itemp);
    if ((itemp=open_file(path, ".index", "r+", INDEX))==NULL)
     /* problem opening index file */
       return(FAIL);

    if (find_problem(itemp, tprid, &prindex)==NULL)
    { /* problem getting original index entry */
       close_file(itemp);
       return(FAIL);
    }
 } /* end problem is a link */
 else /* the problem resides in the leaf given */
 {
    nl_strncpy(path, db_path, DBPATHLEN);
    nl_strncpy(tprid, prid, MAXPRID);
 }
 
 if (!exists(PTSROOT, REOPENDIR, "", "")) /* does the reopen directory */
 {					  /* exist ? */
    make_branch(REOPENDIR); /* no so create it */
    if ((temp=open_file(REOPENDIR, ".index", "r+", INDEX))==NULL)
    { /* problem opening new index */
       close_file(itemp);
       return(FAIL);
    }
/* after creating the reopen directory, we need to write out the number of
 * records that are in the directory, 0 in this case. */
    num_rec=0;
    if (fwrite(&num_rec, sizeof(long), 1, temp)<1)
    {	/* the write failed */
       db_errorno=errno;
       close_file(temp);
       close_file(itemp);
       return(FAIL);
    }
    rewind(temp);
#ifdef DEBUG
 printf("reopen_request() - created reopen directory\n");
#endif
 } /* end reopen dir. does not exist */
 else /* reopen dir exists */
 {
#ifdef DEBUG
 printf("reopen_request() - found reopen directory\n");
#endif

    if ((temp=open_file(REOPENDIR, ".index", "r+", INDEX))==NULL)
    {
       close_file(itemp);
       return(FAIL); /* the open failed */
    }
 } /* end reopen dir. exists */

 if (lock_file(temp)==FAIL) /* lock the reopen index */
 {  /* the lock failed */
    close_file(itemp);
    close_file(temp);
    return(FAIL);
 }
 if (fread(&num_rec, sizeof(long), 1, temp)<1) /* get the number of */
 {   /* records in the index, report failure if read fails */
    db_errorno=errno;
    unlock_file(temp);
    close_file(temp);
    close_file(itemp);
    return(FAIL);
 }
#ifdef DEBUG
printf("reopen_request() - there have been %d reopened requests made\n", num_rec);
#endif

/* we use double problem to check for duplicate problems rather than looking
 * at the mrd field, since if the problem is either a link or an original link
 * the field will not be set correctly
 */
 if (double_problem(temp, path, tprid)) /* check for duplicate reports */
 { /* the problem has already been reported */
    unlock_file(temp);
    close_file(temp);
    close_file(itemp);
    return(FAIL);
 }
#ifdef DEBUG
 printf("reopen_request() - creating reopened index record\n");
#endif

 index.mrd=REOPENED;
 index.sf=UNSOLVED;
 index.ef=FALSE;
 index.account_name[0]=NULL;
 index.node_name[0]=NULL;
 nl_strncpy(index.ln_prid, tprid, MAXPRID);
 nl_strncpy(index.ln_path, path, DBPATHLEN);
 nl_strncpy(index.sd, prindex.sd, MAXSUMLEN); 
 nl_strncpy(index.prid, new_prid(num_rec), MAXPRID);
 if (index.prid[0]==NULL)
 {  /* error creating id */
    unlock_file(temp);
    close_file(temp);
    close_file(itemp);
    return(FAIL);
 }
#ifdef DEBUG
 printf("reopen_request() - created record, now appending it\n");
#endif

 fseek(temp, 0L, SEEK_END); /* move to the end of the reopen index for 
			     * appending */

 if (fwrite(&index, sizeof(index_record), 1, temp)<1)
 { /* error writting to the index */
    db_errorno=errno;
    unlock_file(temp);
    close_file(temp);
    close_file(itemp);
    return(FAIL);
 }
#ifdef DEBUG
printf("reopen_request() - wrote new record, now updating record count\n");
#endif

 /* now go back to the beginning and write out a new record count */
 rewind(temp);
 if (++num_rec>MAXREC) /* we've maxed out the # of possible records */
    num_rec=0; /* hopefully the low numbers have been removed */

 if (fwrite(&num_rec, sizeof(long), 1, temp)<1)
 { /* an error occured while writting */
    unlock_file(temp);
    close_file(temp);
    close_file(itemp);
    delete_record(index.prid);
    return(FAIL);
 }
 /* else the write succeeded so close the reopen index and finish up */
 
 unlock_file(temp);
 close_file(temp);

#ifdef DEBUG
 printf("reopen_request() - updated record count\n");
 printf("reopen_request() - now creating reasons log\n");
#endif

 if (write_log(REOPENDIR, index.prid, reasons)==FAIL)
 { /* error creating reasons log */
    delete_record(index.prid);
    close_file(itemp);
    return(FAIL);
 }

#ifdef DEBUG
 printf("reopen_request() - wrote reasons log\n");
 printf("reopen_request() - creating reporters file\n");
#endif
 
 if ((temp=create_file(REOPENDIR, index.prid, ".rep", FILE_PERMS))==NULL)
 { /* error creating reporters file */
    delete_record(index.prid);
    close_file(itemp);
    return(FAIL);
 }
 if (fwrite(rep_rec, sizeof(reporter_record), 1, temp)<1)
 {	/* error writting reporter record to file */
    db_errorno=errno;
    delete_record(index.prid);
    close_file(temp);
    close_file(itemp);
    return(FAIL);
 }
#ifdef DEBUG
 printf("reopen_request() - created reporter file\n");
#endif

 close_file(temp);

/* since we created the reopened record sucessfully we need to set the mrd
 * field of the problem record, but only if the problem is not an original
 * link (since reopen requests are treated like special cases of original
 * links as far as moving and deleting)
 */

 if (prindex.mrd!=ORGLINK)
 {
    prindex.mrd=REOPENED;

/* since the file pointer is pointing to the end of the index entry we want
 * to overwrite we need to back up one index record.
 */

    fseek(itemp, -(long)(sizeof(index_record)), SEEK_CUR);
    if (fwrite(&prindex, sizeof(index_record), 1, itemp)<1)
    { /* problem writting the new index entry */
       close_file(itemp);
       delete_record(index.prid);
       return(FAIL);
    }
 } /* end !ORGLINK */

/* if we get here everything worked out ok so let the caller know */
 close_file(itemp);
 return(SUCESS);
} /* end reopen_request */


int double_problem(FILE *db_pointer, char *db_path, char *prid)
{
 index_record index;
 int no_doubles=TRUE;

#ifdef DEBUG
 printf("\nIn double_problem()\n");
 printf("\tProblem to search for - %s/%s\n", db_path, prid);
 printf("\tSearching ");
#endif

 while ((no_doubles) &&
		(fread(&index, sizeof(index_record), 1, db_pointer)!=NULL))
 {
#ifdef DEBUG
 printf("?");
#endif

    if ((strcmp(index.ln_path, db_path)==0) &&
		(strcmp(index.ln_prid, prid)==0))
       no_doubles=FALSE;
 }
#ifdef DEBUG
 if (no_doubles)
    printf("\ndouble_problem() - no doubles found\n");
 else
    printf("\ndouble_problem() - found a double\n");
#endif
 if (ferror(db_pointer))
 {	/* there was an i/o error */
    db_errorno=errno;
    return(ERROR);
 }
 else
    if (no_doubles) /* there were no doubles */
       return(FALSE);
    else
    { /* there was a double problem */
       db_errorno=EXISTS;
       return(TRUE);
    } /* end else */
} /* end double_problem */


reopened_record *view_reopen(char *reopen_prid, int whole_record)
{
 static reopened_record reop;
 static long location=sizeof(long);
 char db_path[DBPATHLEN];
 index_record index;
 int size;
 char *log;
 FILE *temp;

#ifdef DEBUG
 printf("\nIn view_reopen()\n");
 printf("\tPrid = %s\n\tWhole record = %d\n", reopen_prid, whole_record);
#endif

  db_errorno=0;

 if ((temp=open_file(REOPENDIR,".index", "r+", INDEX))==NULL)
    return(NULL); /* the open failed */

/* we want to check to see if there any reopen problems before doing anything
 * else, so if file_size() returns a 1 then there are no problems
 */

 if (file_size(temp, sizeof(long))==1)
 { /* no reopen requests */
    db_errorno=NOTFOUND;
    return(NULL);
 }

 if (reopen_prid!=NULL) /* does the caller wants a particular problem */
 {
#ifdef DEBUG
 printf("view_reopen() - looking for a particular problem - %s\n", reopen_prid);
#endif

    if (find_reopen(temp, &index, reopen_prid)==NULL)
    { /* problem finding the record */
       close_file(temp);
       return(NULL);
    } /* end if */
    if (bad_problem(&index, temp))
    { /* the original problem has been deleted */

#ifdef DEBUG
 printf("view_reopen() - problem %s is bad\n", index.prid);
#endif
       close_file(temp);
       if (db_errorno==DELETED)
       {
#ifdef DEBUG
 printf("view_reopen() - problem %s/%s has been deleted\n", index.ln_path, 
		index.ln_prid);
#endif
          delete_record(index.prid);
          location=sizeof(long); /* since we don't know where the problem that
			          * was deleted came from (before or after 
			          * location, we just reset it to the beginning
			          */
       }
       return(NULL);
    }
 } /* end user wants a particular problem */
 else
 { /* the user just wants the next one */
#ifdef DEBUG
 printf("view_reopen() - seeking past last record viewed\n");
#endif

/* seek to the last location looked at.  If that happens to be the end of 
 * file then rewind the pointer and seek past the record counter at the 
 * begining of the file.
 */

    fseek(temp, location, SEEK_SET);

#ifdef DEBUG
 printf("view_reopen() - reading the next record\n");
#endif

/* Now read the index record */

/* the reopen directory is a virtual circular list, when the end is reached
 * you start at the beginning.  The following goto makes the code more
 * "elegant".
 */

    kluge:
    if (fread(&index, sizeof(index_record), 1, temp)==NULL)
    {  /* error reading from file */
       if (feof(temp))
       {
          rewind(temp);
          location=sizeof(long); /* seek past the record counter */
          fseek(temp, location, SEEK_SET);
          goto kluge; /* get the first record */
       }
       else /* it's an error */
       {
          db_errorno=errno;
          close_file(temp);
          return(NULL);
       } /* end error */
    } /* end read */

    if (bad_problem(&index, temp))
    { 
#ifdef DEBUG
 printf("view_reopen() - problem %s is bad\n", index.prid);
#endif

       if (db_errorno=DELETED) /* the original problem has been deleted */
       {
#ifdef DEBUG
 printf("view_reopen() - problem %s/%s has been delted\n", index.ln_path, 
		index.ln_prid);
#endif

/* inorder to remove the offending record from the reopen leaf, we have to 
 * close the file, remove the record and then open it again.  Then since the
 * user just wants the next record we need to get it for them
 */
          close_file(temp);
          delete_record(index.prid);
          if ((temp=open_file(REOPENDIR, ".index", "r+", INDEX))==NULL)
          { /* problem opening index */
	     return(NULL);
          }
	  if (file_size(temp, sizeof(long))==1) /* there are no more records */
	  {
	     db_errorno=NOTFOUND;
	     close_file(temp);
	     return(NULL);
          }
       } /* else it was an error, so skip the record for now */
       else
	  location+=sizeof(index_record);
       fseek(temp, location, SEEK_SET);
       goto kluge;
    } /* end found a bad problem */
#ifdef DEBUG
 printf("view_reopen() - index record read\n");
 print_index(&index);
#endif

/* increment the location pointer */
    location+=sizeof(index_record);
 } /* end else the user didn't want a particular problem */

#ifdef DEBUG
 printf("view_reopen() - creating reopened record\n");
 printf("\tgetting reasons log\n");
#endif

 close_file(temp); /* we don't need the index anymore */
 nl_strncpy(reop.prid, index.prid, MAXPRID); /* the problem id of the 
					   * reopened problem */
 nl_strncpy(reop.db_path, index.ln_path, DBPATHLEN); /* the path to the leaf 
				/* containing the problem to be reopened */

 sprintf(db_path, "%s/%s", REOPENDIR, reop.prid);
 if ((temp=open_file(db_path, ".log", "r", REP))==NULL)
 {	/* error opening reasons log */
    return(NULL);
 }
 size=file_size(temp, sizeof(char));
 if (reop.reasons_log!=NULL) /* free any space allocated for the last log */
    free(reop.reasons_log);

/* since the terminating null character isn't in the log file, we need
 * to append it to the strign we create
 */

 if ((log=(char *)calloc(size+1, sizeof(char)))==NULL)
 {	/* error allocating space for reasons log */
    db_errorno=errno;
    close_file(temp);
    return(NULL);
 }
#ifdef DEBUG
 printf("view_reopen() - allocated space for reasons log\n");
#endif

 if (fread(log, sizeof(char), size, temp)==NULL)
 {	/* error reading reasons log */
    db_errorno=errno;
    close_file(temp);
    return(NULL);
 }
 log[size]='\0'; /* terminate the log with a NULL */
 reop.reasons_log=log;
 close_file(temp);
#ifdef DEBUG
 printf("view_reopen() - read reasons log\n");
 printf("\tgetting reporter record\n");
#endif

 sprintf(db_path, "%s/%s", REOPENDIR, reop.prid);
 if ((temp=open_file(db_path, ".rep", "r", REP))==NULL)
    return(NULL);  /* the open failed */
 if (fread(&reop.rep_rec, sizeof(reporter_record), 1, temp)==NULL)
 {  /* error reading file */
    db_errorno=errno;
    close_file(temp);
    return(NULL);
 }
 close_file(temp);

#ifdef DEBUG
 printf("view_reopen() - got reporter record\n");
 printf("\tgetting problem record to reopen\n");
#endif

/* Now get the original problem */
 if ((reop.pro_rec=get_problem(index.ln_path, index.ln_prid, PARTICULAR, 
				NULL, LINKS, whole_record))==NULL)
    return(NULL); /* there was an error getting the problem record */
 else 
 {
    reop.pro_rec->status=REOPENED; /* the problem is reopened */
    reop.pro_rec->reop_rec=&reop; /* points back to the reopened record */

#ifdef DEBUG
 printf("view_reopen() - changed problem status to REOPENED\n");
 printf("\tstatus - %d\n", reop.pro_rec->status);
#endif

    return(&reop);
 }
} /* end view_reopen */


char *confirm_reopen(char *reop_prid, reporter_record *reop_rec,
		     char *sys_string, char *reasons)
{
 index_record index;
 static index_record org_index;
 FILE *temp,
      *org_temp,
      *temp_two;
 char not_found=TRUE;
 char path[COMPLETEPATHLEN];
 long count,
      bottom;

#ifdef DEBUG
 printf("\nIn confirm_reopen()\n");
 printf("\tPRID to confirm - %s\n\tSysop - %s\n", reop_prid,
		sys_string);
 printf("\tReasons - %s\n", reasons);
#endif

  db_errorno=0;

 if ((temp=open_file(REOPENDIR, ".index", "r+", INDEX))==NULL)
    return(NULL); /* the open failed */
 else
    if (lock_file(temp)==FAIL) /* lock the reopen index file */
    {  /* the lock failed */
       close_file(temp);
       return(NULL);
    }

 if (find_reopen(temp, &index, reop_prid)==NULL)
 {
    unlock_file(temp);
    close_file(temp);
    return(NULL);
 }

#ifdef DEBUG
 printf("\nconfirm_reopen() - found desired problem\n");
 printf("\tnow find original problem record\n");
#endif

/* Open the index for the leaf that holds the problem to be ropened */
 if ((org_temp=open_file(index.ln_path, ".index", "r+", INDEX))==NULL)
 {  /* could not open the index file */
    unlock_file(temp);
    close_file(temp);
    return(NULL);
 }

/* Now find the problem to be reopend */
 if (find_problem(org_temp, index.ln_prid, &org_index)==NULL)
 {  /* error looking for the problem */
    unlock_file(temp);
    close_file(temp);
    close_file(org_temp);
    return(NULL);
 }
#ifdef DEBUG
 printf("confirm_reopen() - found original problem\n");
 printf("\tnow creating the new reporter file\n");
#endif

/* create the new reporters list and add the new reporters record */
 if ((temp_two=create_file(index.ln_path, org_index.prid, ".rep",
			   FILE_PERMS))==NULL)
 {  /* error creating new file */
    unlock_file(temp);
    close_file(temp);
    close_file(org_temp);
    return(NULL);
 }
 if (fwrite(reop_rec, sizeof(reporter_record), 1, temp_two)<1)
 {  /* error writting to the file */
    db_errorno=errno;
    unlock_file(temp);
    close_file(temp);
    close_file(org_temp);
    close_file(temp_two);
    remove_file(index.ln_path, org_index.prid, ".rep");
    return(NULL);
 }
 else
    close_file(temp_two);

#ifdef DEBUG
 printf("confirm_reopen() - created new reporter file\n");
 printf("\tappending sysop string to log\n");
#endif

/* Now add the name of the person confirming the reopen (sys_string) and
 * the reasons for the reopening to the problem log, but first get the position
 * of the current end of file.  This way if an error occurs before completing
 * the reopening the additions can be removed (at least in theory).
 */
 sprintf(path, "%s/%s", index.ln_path, org_index.prid);
 if ((temp_two=open_file(path, ".log", "w", REP))==NULL)
 {  /* error opening log file */
    unlock_file(temp);
    close_file(temp);
    close_file(org_temp);
    remove_file(index.ln_path, org_index.prid, ".rep");
    return(NULL);
 }
/* seek to the bottom of the file before appending */
 fseek(temp_two, 0L, SEEK_END);
 bottom=ftell(temp_two); /* get the position of the end of the file */

 if (fwrite(sys_string, sizeof(char), strlen(sys_string), temp_two)
		<strlen(sys_string))
 { /* error writting sys_string to log */
    db_errorno=errno;
    unlock_file(temp);
    close_file(temp);
    close_file(org_temp);
    close_file(temp_two);
    remove_file(index.ln_path, org_index.prid, ".rep");
    return(NULL);
 }

#ifdef DEBUG
 printf("confirm_reopen() - wrote sysop string\n");
 printf("\tappending reasons\n");
#endif

 if (fwrite(reasons, sizeof(char), strlen(reasons), temp_two)
		<strlen(reasons))
 {  /* error writting to the log file */
    db_errorno=errno;
    unlock_file(temp);
    close_file(temp);
    close_file(org_temp);
    close_file(temp_two);
    trunc_file(path, ".log", bottom); /* cut off the extra stuff */
    remove_file(index.ln_path, org_index.prid, ".rep");
    return(NULL);
 }

#ifdef DEBUG
 printf("confirm_reopen() - appended reasons\n");
 printf("\tnow update the index entry\n");
#endif

/* now lock the index file and write out a new index entry for the reopened
 * problem.
 */

 if (lock_file(org_temp)==FAIL)
 {  /* the locking failed */
    unlock_file(temp);
    close_file(temp);
    close_file(org_temp);
    close_file(temp_two);
    trunc_file(path, ".log", bottom);
    remove_file(index.ln_path, org_index.prid, ".rep");
    return(NULL);
 }

 org_index.sf=UNSOLVED;
 if ((org_index.mrd!=ORGLINK) || (org_index.mrd!=LINKED)
     || (org_index.mrd!=MOVED))
    org_index.mrd=0;
 fseek(org_temp, -(long)sizeof(index_record), SEEK_CUR);
 if (fwrite(&org_index, sizeof(index_record), 1, org_temp)<1)
 {  /* error updateing index */
    db_errorno=errno;
    unlock_file(temp);
    unlock_file(org_temp);
    close_file(temp);
    close_file(org_temp);
    close_file(temp_two);
    trunc_file(path, ".log", bottom);
    remove_file(index.ln_path, org_index.prid, ".rep");
    return(NULL);
 }
#ifdef DEBUG
 printf("confirm_reopen() - updated index entry\n");
 printf("\tnow remove the reopen record\n");
#endif
 /* now unlock and close everything, then remove the entry from the reopend
  * directory.  We are not overly concerned if the record removal fails, since
  * the update has occured.
  */

  unlock_file(org_temp);
  close_file(org_temp);
  unlock_file(temp);
  close_file(temp);
  close_file(temp_two);
  if (delete_record(reop_prid)==FAIL)
  {
#ifdef DEBUG
  printf("deletetion failed,errno=%d\n",errno) ;
#endif
  	/* deletion failed */
     return(NULL);
  }
  else
     return(org_index.prid);
} /* end confirm_reopen */


int deny_reopen(char *reop_prid)
{
 FILE *temp;
 index_record index;
 char tprid[MAXPRID],
      tpath[DBPATHLEN];

#ifdef DEBUG
 printf("\nIn deny_reopen()\n");
 printf("\tPRID - %s.\n", reop_prid);
#endif

  db_errorno=0;

 if ((temp=open_file(REOPENDIR, ".index", "r", INDEX))==NULL)
 { /* problem opening reopen index */
    return(FAIL);
 }

 if (find_reopen(temp, &index, reop_prid)==NULL)
 { /* problem finding reopen record */
    close_file(temp);
    return(FAIL);
 }

#ifdef DEBUG
 printf("deny_reopen() - found reopen index record desired\n");
#endif

 close_file(temp); /* we don't need the reopen index anymore */
 nl_strncpy(tprid, index.ln_prid, MAXPRID);
 nl_strncpy(tpath, index.ln_path, DBPATHLEN);

#ifdef DEBUG
 printf("deny_reopen() - now reset the original problem mrd to 0\n");
#endif

/* now open the index the original problem resides in, so we can set the
 * mrd field to 0 (the problem is no longer requested for reopening.
 */

 if ((temp=open_file(tpath, ".index", "r+", INDEX))==NULL)
    return(FAIL);

 if (find_problem(temp, tprid, &index)==NULL)
 { /* error finding problem */
    close_file(temp);
    return(FAIL);
 }

/* we've got the index entry, so reset the mrd field and write the new index
 * entry out, after resetting the file pointer to point to the begining of the
 * entry we want to overwrite.  Also don't reset the mrd field if it's already
 * set ot LINKED, ORGLINK or MOVED
 */

 if (lock_file(temp)==FAIL) /* lock the index before updating it */
 {
    close_file(temp);
    return(FAIL);
 }
 if ((index.mrd!=ORGLINK)  && (index.mrd!=MOVED) && (index.mrd!=LINKED))
 { /* we can reset the mrd field */
    index.mrd=0;
    fseek(temp, -(long)sizeof(index_record), SEEK_CUR);
    if (fwrite(&index, sizeof(index_record), 1, temp)<1)
    { /* problem writting the index entry */
       unlock_file(temp);
       close_file(temp);
       return(FAIL);
    }
    unlock_file(temp);
    close_file(temp);
 }
 else
    close_file(temp);

 return(delete_record(reop_prid));
} /* end deny_reopen */


int delete_record(char *reop_prid)
{
 index_record index;
 char *iname;
 char tfile[FNAMELEN];
 FILE *temp,
      *temp_two;
 long num_rec;

#ifdef DEBUG
 printf("\nIn delete_record()\n\tRecord to remove - %s\n", reop_prid);
#endif

  nl_strncpy(tfile, TEMPFILE, FNAMELEN);

 if ((temp=open_file(REOPENDIR, ".index", "r", INDEX))==NULL)
    return(FAIL);
#ifdef DEBUG
 printf("\tdelete_record() - temp=%p,(file descriptor=%d)\n",temp,fileno(temp)) ;
#endif
 if (lock_file(temp)==FAIL)
 {  /* the lock of the index failed */
    close_file(temp);
    return(FAIL);
 }

#ifdef DEBUG
 printf("delete_record() - creating temporary file\n");
#endif
 if ((temp_two=create_file(REOPENDIR, (char *)mktemp(tfile), NULL,
			   FILE_PERMS))==NULL)
 {  /* the creation failed */
    unlock_file(temp);
    close_file(temp);
    return(FAIL);
 }
 if (lock_file(temp_two)==FAIL)
 {  /* the lock the temp file failed */
    unlock_file(temp);
    close_file(temp);
    close_file(temp_two);
    remove_file(REOPENDIR, tfile, NULL);
    return(FAIL);
 }

/* we need to read the record count from the index file and write it to the
 * new index file before searching for the records. */

 fread(&num_rec, sizeof(long), 1, temp);
 fwrite(&num_rec, sizeof(long), 1, temp_two);

#ifdef DEBUG
 printf("delete_record() - searching for record to remove ");
#endif

 while (fread(&index, sizeof(index_record), 1, temp)==1)
 {
#ifdef DEBUG
 printf("?");
#endif

    if (strcmp(index.prid, reop_prid)!=0)
       if (fwrite(&index, sizeof(index_record), 1, temp_two)<1)
	  break; /* the write failed */
 } /* end while */

 if (ferror(temp) || ferror(temp_two)) /* did we leave the loop because of an 
					* error reading or writting? */
 {
    db_errorno=errno;
    unlock_file(temp);
    close_file(temp);
    unlock_file(temp_two);
    close_file(temp_two);
    remove_file(REOPENDIR, tfile, NULL);
    return(FAIL);
 }
#ifdef DEBUG
 printf("\ndelete_record() - removed record from index\n");
 printf("\trenaming temp file\n");
#endif

/* now that we've removed the offending index record, delete the old index
 * and rename the temp file, so it will become the new index.  
 */

 iname=(char *)rindex(REOPENDIR, '/');
 if (iname==NULL)
    iname=REOPENDIR; /* the name of the index file is the name of the dir */
 else
    iname++; /* want char after '/' */
 unlock_file(temp);
 unlock_file(temp_two);
 close_file(temp);
 close_file(temp_two);
 rename_file(REOPENDIR, tfile, "", REOPENDIR, iname, ".index");
 remove_file(REOPENDIR, reop_prid, ".rep");
 remove_file(REOPENDIR, reop_prid, ".log");
 return(SUCESS);
} /* end delete_record */


index_record *find_reopen(FILE *db_pointer, index_record *index, char *prid)
{

#ifdef DEBUG
 printf("In find_reopen()\n");
 printf("\tsearching for %s/%s\n", REOPENDIR, prid);
#endif

/* call search_index() to look for the particular problem */

 return(search_index(db_pointer, prid, index, sizeof(long)));
} /* end find_reopen() */
 
/* now create the reopened record, but before doing that check to make sure
 * the original problem hasn't been deleted or moved.  if it's been deleted
 * then remove the entry from the reopen leaf, if its been moved then update
 * the link fields
 */

int bad_problem(index_record *index, FILE *db_pointer)
{
 index_record prindex;
 FILE *intemp;

#ifdef DEBUG
 printf("In bad_problem()\n");
 printf("\tProblem to look for %s/%s\n", index->ln_path, index->ln_prid);
#endif

 if ((intemp=open_file(index->ln_path, ".index", "r", INDEX))==NULL)
 { /* problem opening the problem index */
    return(ERROR);
 }
 if (find_problem(intemp, index->ln_prid, &prindex)==NULL)
 { /* problem finding the index record */
    close_file(intemp);
    if (db_errorno==DELETED)
    { /* the problem has been deleted */
#ifdef DEBUG
 printf("bad_problem() - original problem has been deleted\n");
#endif
       return(TRUE);
    }
    else
       return(ERROR);
 } /* end problem with index */
 close_file(intemp); /* we don't need it now */

/* found the index record now update the reopen index record, if needed */

 if (prindex.mrd==MOVED)
 { /* the problem was moved, so update the reopen record */

#ifdef DEBUG
 printf("bad_problem() - original problem has been moved - updating\n");
#endif
    nl_strncpy(index->ln_path, prindex.ln_path, DBPATHLEN);
    nl_strncpy(index->ln_prid, prindex.ln_prid, MAXPRID);

    fseek(db_pointer, -(long)sizeof(index_record), SEEK_CUR);
    if (fwrite(index, sizeof(index_record), 1, db_pointer)<1)
    {
       db_errorno=errno;
       return(ERROR);
    }
    else
    {
#ifdef DEBUG
 printf("bad_problem() - reopen index entry has been updated\n");
#endif
       return(FALSE);
    }
 } /* end == MOVED */
 else /* the origial proble has no problems */
 {
#ifdef DEBUG
 printf("bad_problem() - problem is good\n");
#endif
 
    return(FALSE);
 } /* end else no problems */
} /* end bad_problem() */

int is_reopened(char *path, char *prid)
{
 FILE *temp;
 index_record index;
 char tprid[MAXPRID],
      tpath[DBPATHLEN];
 int result;

#ifdef DEBUG
 printf("In is_reopened()\n");
 printf("Checking for problem %s/%s\n", path, prid);
#endif

 db_errorno=0;

/* since the index entries of the REOPEN index point back to the original
 * problem in a series of linked problems, we need to get the index of
 * the problem to check and get the path to the origianl problem if it's
 * linked, if it's not then we use the path and prid passed in
 */

 if ((temp=open_file(path, ".index", "r+", INDEX))==NULL)
    return(ERROR); /* problem opening index file */

 if (find_problem(temp, prid, &index)==NULL)
 {
    close_file(temp);
    return(ERROR);
 }
 close_file(temp); /* we don't need to keep the index open after getting the
		    * index entry */

 if (index.mrd==LINKED)
 { /* we have different paths and prids for linked problems */
    nl_strncpy(tprid, index.ln_prid, MAXPRID);
    nl_strncpy(tpath, index.ln_path, DBPATHLEN);
 }
 else /* the problem isn't linked */
 {
    nl_strncpy(tprid, prid, MAXPRID);
    nl_strncpy(tpath, path, DBPATHLEN);
 }

 if ((temp=open_file(REOPENDIR, ".index", "r", INDEX))==NULL)
 { /* problem opening the reopen index */
    return(ERROR);
 }
 else
 {
    result=double_problem(temp, path, prid);
    close_file(temp);
    return(result);
 } /* end else */
} /* end isreopened() */
/* end of reopen_problem.c */
